'''
% Chow Test
% This program conducts chow test with log-cumulative hazard data.
%
% This program requires R, rpy, numpy, surv2sample (R package), and chow test program.
%
%    R ==> http://www.r-project.org/
%    rpy ==> http://rpy.sourceforge.net/
%    numpy ==> http://numpy.scipy.org/
%    surv2sample (R package) ==> http://cran.r-project.org/web/packages/surv2sample/index.html
%
%    chow.py, lm.py, matlib.py functions were implemented by Dr. Ernesto P. Adorio in http://adorio-research.org/
%
%    chow.py ==> http://adorio-research.org/wordpress/?p=1789
%    lm.py ==> http://adorio-research.org/wordpress/?p=1932
%    matlib.py ==> http://adorio-research.org/wordpress/?p=1930
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

import chow
import SurvivalData

def ChowTest( aData1, aData2 ):
    return chow.chow_test( aData1.timelist_mot, aData1.mortality, aData2.timelist_mot, aData2.mortality )

def RunChowTestAll( conditions ):
    results_chowTest = []

    for i in range( 0, len( conditions ) ):
        aData1 = conditions[i]
        for j in range( 0, len( conditions ) ):
            if i == j:
                continue
            aData2 = conditions[j]
            P_value, F_value, Fcrit, df1, df2, RSS, RSS1, RSS2 = ChowTest( aData1, aData2 )
            Corrected_P_value = min( P_value * ( len( conditions ) - 1 ), 1.0 ) # Bonferroni Correction

            results_chowTest.append( [ aData1.name, aData2.name, F_value, df1, df2, P_value, Corrected_P_value ] )
    return results_chowTest


if __name__ == "__main__":    
    conditions = SurvivalData.ReadDataFromFile( "sample.txt" )
    SurvivalData.CalculateSubplotData( conditions )
    
    results_chowTest = RunChowTestAll( conditions )

    print "\t".join( [ "Sample 1", "Sample 2", "F-value", "DF1", "DF2", "P-value", "Corrected P-value" ] )
    for results in results_chowTest:
        for v in results:
            print v, "\t",
        print ""




