'''
% Fisher's Exact Test
% This program provides statistical test for comparing different survival functions at specific time point instead of overall lifespan.
%
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

import SurvivalData
import sys, os

def getCommandOutput(command):
    child = os.popen(command)
    data = child.read()
    err = child.close()
    if err:
        raise RuntimeError, '%s failed w/ exit code %d' % (command, err)
    return data

def getFisherPvalue( a, b, c, d ):
    if sys.platform=="win32":
        command = "fisherexact.exe %d %d %d %d 3" % ( a,b,c,d )
        pvalue = getCommandOutput( command )
    else:
        command = "./fisherexact %d %d %d %d 3" % ( a,b,c,d )
        pvalue = getCommandOutput( command )

    #from scipy import stats
    #[x, pvalue] = stats.fisher_exact( [[a,b],[c,d]] )
    #if pvalue < 0:
    #    [x, pvalue] = stats.fisher_exact( [[b,a],[d,c]] )
    #print a,b,c,d,pvalue
    
    return float( pvalue )

def FisherExactTest( aData1, aData2, mortality ):
    data1_dead = 0 
    data2_dead = 0 
    Risk = aData1.survall + aData2.survall

    timelist = list( set( aData1.timelist + aData2.timelist ) )
    timelist.sort()

    for index in timelist:
        data1_dead += aData1.deadlist.get( index, 0 )
        data2_dead += aData2.deadlist.get( index, 0 )

        dead_percent = ( data1_dead + data2_dead ) / float( Risk )

        if dead_percent > mortality:
            data1_alive = aData1.all - data1_dead
            data2_alive = aData2.all - data2_dead

            return getFisherPvalue( data1_alive, data2_alive, data1_dead, data2_dead )

    return 2 
    
def FisherTestAll( conditions ):
    results_FisherTest = []

    for i in range( 0, len( conditions ) ):
        aData1 = conditions[i]
        for j in range( 0, len( conditions ) ):
            if i == j:
                continue
            aData2 = conditions[j]

            P_value25 = FisherExactTest( aData1, aData2, 0.25 )
            P_value50 = FisherExactTest( aData1, aData2, 0.5 )
            P_value75 = FisherExactTest( aData1, aData2, 0.75 )
            P_value90 = FisherExactTest( aData1, aData2, 0.9 )

            Corrected_P_value25 = min( P_value25 * ( len( conditions ) - 1 ), 1.0 )
            Corrected_P_value50 = min( P_value50 * ( len( conditions ) - 1 ), 1.0 )
            Corrected_P_value75 = min( P_value75 * ( len( conditions ) - 1 ), 1.0 )
            Corrected_P_value90 = min( P_value90 * ( len( conditions ) - 1 ), 1.0 )

            results_FisherTest.append( [ aData1.name, aData2.name, P_value25, P_value50, P_value75, P_value90, \
                                         Corrected_P_value25, Corrected_P_value50, Corrected_P_value75, Corrected_P_value90 ] )

    return results_FisherTest    

if __name__ == "__main__":
    conditions = SurvivalData.ReadDataFromFile( "sample.txt" )

    results_logRankTest = FisherTestAll( conditions )

    print "\t".join( ["Sample 1", "Sample 2", "P-value at 25%", "P-value at 50%", "P-value at 75%", "P-value at 90%", \
                      "Corrected P-value at 25%", "Corrected P-value at 50%", "Corrected P-value at 75%", "Corrected P-value at 90%"] )
    for results in results_logRankTest:
        for v in results:
            print v, "\t",
        print ""

        
