'''
% Partial slopes Rank Sum Test
% This program conducts partial slopes Rank Sum test for comparing the difference of slope of two log cumulative hazard plots.
%
% This program requires R, rpy, numpy
%    R ==> http://www.r-project.org/
%    rpy ==> http://rpy.sourceforge.net/
%    numpy ==> http://numpy.scipy.org/
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

import rpy
import numpy
import SurvivalData

def PartialSlopesRankSumTest( aData1, aData2 ):
    mot_slope_list1 = [ ]

    for i in range( 1, len( aData1.timelist_mot ) ):
        mot_slope_list1.append( (aData1.mortality[i-1] - aData1.mortality[i]) / ( aData1.timelist_mot[i-1] - aData1.timelist_mot[i] ) )

    mot_slope_list2 = [ ]
    for i in range( 1, len( aData2.timelist_mot ) ):
        mot_slope_list2.append( (aData2.mortality[i-1] - aData2.mortality[i]) / ( aData2.timelist_mot[i-1] - aData2.timelist_mot[i] ) )

    result = rpy.r.wilcox_test( mot_slope_list1, mot_slope_list2 )
    slope_ratio = numpy.average( mot_slope_list1 ) / numpy.average( mot_slope_list2 )
    
    pvalue = result['p.value']

    return slope_ratio, pvalue

def RunPartialSlopesRankSumTestAll( conditions ):
    results_var2Test = []

    for i in range( 0, len( conditions ) ):
        aData1 = conditions[i]
        for j in range( 0, len( conditions ) ):
            if i == j:
                continue
            aData2 = conditions[j]

            var_ratio, P_value = PartialSlopesRankSumTest( aData1, aData2 )
            Corrected_P_value = min( P_value * ( len( conditions ) - 1 ), 1.0 )
            
            results_var2Test.append( [ aData1.name, aData2.name, var_ratio, P_value, Corrected_P_value ] )

    return results_var2Test


if __name__ == "__main__":    
    conditions = SurvivalData.ReadDataFromFile( "sample.txt" )
    SurvivalData.CalculateSubplotData( conditions )

    results_var2Test = RunPartialSlopesRankSumTestAll( conditions )

    print "\t".join( [ "Sample 1", "Sample 2", "Ratio of Slopes", "P-value", "Corrected P-value" ] )
    for results in results_var2Test:
        for v in results:
            print v, "\t",
        print ""
